// A script to get the engine moving around a bit. Done in as realistic manner as possible without getting really deep for no reason.
// It is this mechanic in real life that gives us interesting driveline effects like axle hop etc, due to the fluctuating torque output at the wheels as engines
// act at high loads against the engine mounts. Ie, mounts absorb torque, we are near limit of traction, mounts reach full deflection, extra torque spins wheels,
// slip ratio builds up, grip is reduced, wheel spins more as extra torque is released from engine mounts, engine bounces back, torque reduces, traction is regained,
// and so the process starts again!
// Racer needs it's engine like an SMD system, then we could just link the model to the in-sim variable for engine rotation, also we would get nice drivetrain characteristics,
// like the engine 'jitter effect', but based on real variables with real effects!
// Hint hint Ruud :-D

rcar $car = get local car

rcarmodel $wobble = get generic model 10 of $car
set $wobble scriptcontrolled 1

float $engine_load
float $engine_twist
float $engine_output_torque
float $rpm
float $engine_rot = 0


// How hard the torque tries to turn the engine
float $engine_coefficient = 1.75

// How fast the engine returns back to vertical (real effect fudged a bit, needs to be tied into Racer physics, the engine needs to be an SMD really)
float $return_rate = 0.01

// Max engine rotation, hard limit
float $max_rot = 0.12

// Damping that slows the rate of change of the engine rotation
float $damping = 0.003

func void wobble()
{
	$engine_load = get system "car0.engine_load"
	$rpm = get $car rpm
	
	// No access to engines current torque output, so generating it (roughly) from a quadratic vs rpm
	$engine_output_torque = (-(0.00001 * (pow $rpm to 2)) + (0.1243 * $rpm) + 140) * $engine_load * $engine_coefficient
		
	// How much angular twisting the engine generates, engine torque divided by a coefficient
	$engine_twist = $engine_load * ($engine_output_torque / 18000)
	
	// So once the engine is twisted on it's mounts, if the force subsides that is twisting it, the energy in the mounts is released via this variable, also hard rotational limits are set here
	if $engine_twist > 0
	{
		$engine_twist -= $return_rate
		
		if $engine_twist > $max_rot
		{
			$engine_twist = $max_rot
		}
	}
	
	// And this limits the rate at which the engine rotating on it's mounts can take on and release energy
	if $engine_rot > $engine_twist
	{
		$engine_rot -= $damping
	}
	
	if $engine_rot < $engine_twist
	{
		$engine_rot += $damping
	}
	
	// Make our object wobble!!! Note it's negative engine rot variable, thats because Racer body torque reaction is only in one direction, so I had to flip it
	// to make it look correct (engine going opposite way to the body)
	set $wobble rotation float[3]{0,0,-$engine_rot}
}

while 1
{
	wobble()
	interrupt
}